# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/superpower.chisq.ipynb (unless otherwise specified).

__all__ = ['ChiSqInd', 'ChiSqCon']

# Cell
from ..rpy import RFunction
import numpy as np

# Cell
class ChiSqInd(RFunction):

    argNames = ['n_row', 'n_col', 'N', 'ns_cells', 'pVal', 'w']

    def __init__(self, argNames=argNames, **kwargs):
        super().__init__(name='powerChiSqInd', argNames=argNames, **kwargs)

class ChiSqCon(RFunction):

    argNames=['pVal', 'mata', 'matb', 'rcomp']

    def __init__(self, argNames=argNames, **kwargs):
        super().__init__(name='powerChiSqCont', argNames=argNames, **kwargs)