# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/views.chisq.ipynb (unless otherwise specified).

__all__ = ['ChiSqLanding', 'ChiSq']

# Cell
from .ratio import RatioLanding, RatioEstimates
from ..widgets import Textarea, Button
from ..widget_containers import  HBox, VBox
from ..models.chisq import ChiSqModel
from ..mvc import Controller
from .corr import CramerV

# Cell
import ipywidgets as ipyw

# Cell
class ChiSqLanding(RatioLanding):
    pass

# Cell
class ChiSq(CramerV):

    def __init__(self, model, **kwargs):
        super().__init__(model, **kwargs)