# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/views.tabs.ipynb (unless otherwise specified).

__all__ = ['LandingTab', 'EstimatesTab', 'ResultsTab', 'SettingsTab']

# Cell
from ..widget_containers import VBox, HBox

# Cell
class LandingTab(VBox):
    def __init__(self, dropdown, landing_box, okay_button, **kwargs):
        super().__init__(**kwargs)
        self.children = (dropdown, landing_box, okay_button)

# Cell
class EstimatesTab(VBox):
    def __init__(self, estimates_box, run_button, **kwargs):
        super().__init__(**kwargs)
        self.children = (estimates_box, run_button)

# Cell
class ResultsTab(VBox):
    def __init__(self, reset_button, repopulate_landing, repopulate_estimates, resultsView, **kwargs):
        super().__init__(**kwargs)
        hbox = HBox()
        hbox.children = (reset_button, repopulate_landing, repopulate_estimates)
        self.children = (resultsView, hbox)

# Cell
class SettingsTab(VBox):
    def __init__(self, settingsView, log_widget, **kwargs):
        super().__init__(**kwargs)
        self.children = (settingsView, log_widget)