# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"Item": "00_items.ipynb",
         "Collection": "00_items.ipynb",
         "Selection": "controller.ipynb",
         "ItemList": "00_items.ipynb",
         "ItemArray": "00_items.ipynb",
         "Widget": "01_widgets.ipynb",
         "Output": "01_widgets.ipynb",
         "IntSlider": "01_widgets.ipynb",
         "FloatSlider": "01_widgets.ipynb",
         "IntProgress": "01_widgets.ipynb",
         "BoundedIntText": "01_widgets.ipynb",
         "BoundedFloatText": "01_widgets.ipynb",
         "IntText": "01_widgets.ipynb",
         "FloatText": "01_widgets.ipynb",
         "ToggleButtons": "01_widgets.ipynb",
         "Checkbox": "01_widgets.ipynb",
         "Dropdown": "01_widgets.ipynb",
         "RadioButtons": "01_widgets.ipynb",
         "Select": "01_widgets.ipynb",
         "Text": "01_widgets.ipynb",
         "Textarea": "01_widgets.ipynb",
         "Label": "01_widgets.ipynb",
         "HTML": "01_widgets.ipynb",
         "HTMLMath": "01_widgets.ipynb",
         "Image": "01_widgets.ipynb",
         "Button": "01_widgets.ipynb",
         "ColorPicker": "01_widgets.ipynb",
         "Layout": "01_widgets.ipynb",
         "HBox": "02_widget_containers.ipynb",
         "AlignedLabel": "01_widgets.ipynb",
         "IntLabel": "01_widgets.ipynb",
         "Grid": "01_widgets.ipynb",
         "WidgetContainer": "02_widget_containers.ipynb",
         "Box": "02_widget_containers.ipynb",
         "VBox": "02_widget_containers.ipynb",
         "WidgetList": "02_widget_containers.ipynb",
         "WidgetRow": "02_widget_containers.ipynb",
         "WidgetColumn": "02_widget_containers.ipynb",
         "SelectionContainer": "02_widget_containers.ipynb",
         "SelectionBox": "02_widget_containers.ipynb",
         "SelectionVBox": "02_widget_containers.ipynb",
         "SelectionHBox": "02_widget_containers.ipynb",
         "TabAcc": "02_widget_containers.ipynb",
         "Tab": "02_widget_containers.ipynb",
         "Accordion": "02_widget_containers.ipynb",
         "APP_DIR": "03_config.ipynb",
         "CONFIG": "03_config.ipynb",
         "USER_DIR": "03_config.ipynb",
         "DispatchingFormatter": "04_log.ipynb",
         "WidgetHandler": "04_log.ipynb",
         "Handlers": "04_log.ipynb",
         "HANDLERS": "04_log.ipynb",
         "logger": "views.table_widget.ipynb",
         "RConsole": "05_rpy.ipynb",
         "R_CONSOLE": "05_rpy.ipynb",
         "rpy2": "results.ipynb",
         "lib_loc": "05_rpy.ipynb",
         "lib_paths": "05_rpy.ipynb",
         "pkg_name": "05_rpy.ipynb",
         "utils": "05_rpy.ipynb",
         "types": "05_rpy.ipynb",
         "DTYPES": "05_rpy.ipynb",
         "AS_DTYPE": "05_rpy.ipynb",
         "Result": "05_rpy.ipynb",
         "RFunction": "05_rpy.ipynb",
         "RMixins": "05_rpy.ipynb",
         "IntArray": "06_traits.ipynb",
         "FloatArray": "06_traits.ipynb",
         "Int": "06_traits.ipynb",
         "Float": "06_traits.ipynb",
         "LinkAllMixin": "06_traits.ipynb",
         "Model": "07_mvc.ipynb",
         "Controller": "controller.ipynb",
         "MiniView": "07_mvc.ipynb",
         "ChiSqLandingModel": "models.chisq.ipynb",
         "ChiSqModel": "models.chisq.ipynb",
         "CramerVLandingModel": "models.corr.ipynb",
         "AddStringsMixin": "models.corr.ipynb",
         "CramerVModel": "models.corr.ipynb",
         "TetraLandingModel": "models.corr.ipynb",
         "TetraModel": "models.corr.ipynb",
         "CorrLandingModel": "models.corr.ipynb",
         "CorrModel": "models.corr.ipynb",
         "SpearmansRhoModel": "models.corr.ipynb",
         "PearsonModel": "models.corr.ipynb",
         "PBiModel": "models.corr.ipynb",
         "Ratios": "models.ratio.ipynb",
         "RatioArray": "models.ratio.ipynb",
         "RatioLandingModel": "models.ratio.ipynb",
         "RatioEstimatesModel": "models.ratio.ipynb",
         "ResultsModel": "results.ipynb",
         "ResultsView": "results.ipynb",
         "Results": "results.ipynb",
         "SettingsModel": "settings.ipynb",
         "SettingsView": "settings.ipynb",
         "Settings": "settings.ipynb",
         "ChiSqInd": "superpower.chisq.ipynb",
         "ChiSqCon": "superpower.chisq.ipynb",
         "CorrPearson": "superpower.corr.ipynb",
         "CorrPBi": "superpower.corr.ipynb",
         "CorrSpearmansRho": "superpower.corr.ipynb",
         "CorrCramerV": "superpower.corr.ipynb",
         "CorrTetra": "superpower.corr.ipynb",
         "ChiSqLanding": "views.chisq.ipynb",
         "ChiSq": "views.chisq.ipynb",
         "CramerVLanding": "views.corr.ipynb",
         "CramerVEstimates": "views.corr.ipynb",
         "CramerV": "views.corr.ipynb",
         "TetraLanding": "views.corr.ipynb",
         "Tetra": "views.corr.ipynb",
         "CorrLanding": "views.corr.ipynb",
         "SpearmansRhoLanding": "views.corr.ipynb",
         "PBiLanding": "views.corr.ipynb",
         "PearsonLanding": "views.corr.ipynb",
         "CorrView": "views.corr.ipynb",
         "Corr": "views.corr.ipynb",
         "SpearmansRho": "views.corr.ipynb",
         "PBi": "views.corr.ipynb",
         "Pearson": "views.corr.ipynb",
         "RatioLandingView": "views.ratio.ipynb",
         "RatioLanding": "views.ratio.ipynb",
         "ColorInt": "views.ratio.ipynb",
         "RatioEstimatesView": "views.ratio.ipynb",
         "RatioEstimates": "views.ratio.ipynb",
         "Table": "views.table_widget.ipynb",
         "LandingTab": "views.tabs.ipynb",
         "EstimatesTab": "views.tabs.ipynb",
         "ResultsTab": "views.tabs.ipynb",
         "SettingsTab": "views.tabs.ipynb"}

modules = ["items.py",
           "widgets.py",
           "widget_containers.py",
           "config.py",
           "log.py",
           "rpy.py",
           "traits.py",
           "mvc.py",
           "controller.py",
           "models/chisq.py",
           "models/corr.py",
           "models/ratio.py",
           "results.py",
           "settings.py",
           "superpower/chisq.py",
           "superpower/corr.py",
           "views/chisq.py",
           "views/corr.py",
           "views/ratio.py",
           "views/table_widget.py",
           "views/tabs.py"]

doc_url = "https://pages.github.itap.purdue.edu/brewer36/superpower_gui/"

git_url = "https://github.itap.purdue.edu/superpower_gui/superpower_gui/tree/master/"

def custom_doc_links(name): return None
