# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/superpower.corr.ipynb (unless otherwise specified).

__all__ = ['CorrPearson', 'CorrPBi', 'CorrSpearmansRho', 'CorrCramerV', 'CorrTetra']

# Cell
import numpy as np

# Cell
from ..log import HANDLERS
from ..rpy import RFunction, R_CONSOLE

# Cell
class CorrPearson(RFunction): # one
    argNames = ['N', 'r', 'alternative', 'pVal']
    def __init__(self, argNames=argNames, **kwargs):
        super().__init__(name='corrPearson', argNames=argNames, **kwargs)

class CorrPBi(RFunction): # two
    argNames = ['N', 'r', 'tail', 'pVal']
    def __init__(self, argNames=argNames, **kwargs):
        super().__init__(name='corrPBi', argNames=argNames, **kwargs)

class CorrSpearmansRho(RFunction): # one
    argNames = ['N', 'r', 'alternative', 'pVal']
    def __init__(self, argNames=argNames, **kwargs):
        super().__init__(name='corrSpearmansRho', argNames=argNames, **kwargs)

class CorrCramerV(RFunction): # one
    argNames=['n_row', 'n_col', 'alternative', 'ns_cells', 'N', 'pVal']
    def __init__(self, argNames=argNames, **kwargs):
        super().__init__(name='corrCramerV', argNames=argNames, **kwargs)

class CorrTetra(RFunction): # two
    argNames=['n_row', 'n_col', 'tail', 'ns_cells', 'N', 'pVal']
    def __init__(self, argNames=argNames, **kwargs):
        super().__init__(name='corrTetra', argNames=argNames, **kwargs)